# coffeelint: disable=max_line_length

fs = require("fs")
spawn = require("child_process").spawn
exec = require("child_process").exec

async = require 'async'
_ = require 'lodash'

logger = require("../../../shared_modules/logger")

Producer = require "../../ProducerClass"

class Gatelogs extends Producer
  isRunning = false

  logConfigPath = "/etc/rsyslog.d/50-default.conf"
  webLogConfigPath = "/etc/rsyslog.d/51-weblog.conf"
  logOutputFilePath = "/var/log/omgweblog/webSyslog.olg"

  writeNewConfig = (parallelCallback) ->
    ### read config, replace path and write it to new config ###
    # read logging config and split into lines
    originalLogConfig = (fs.readFileSync logConfigPath).toString()
    originalLogConfigLines = originalLogConfig.split("\n")
    #line 3 contains the path that has to be changed
    pathToReplace = _.last(originalLogConfigLines[2].split " ")
    pathToReplace = pathToReplace.split(";")[0]

    #set new config with different path
    regex = new RegExp(pathToReplace, 'g')
    newConfig = originalLogConfig.replace(regex, logOutputFilePath)

    fs.writeFile webLogConfigPath, newConfig, (err) ->
      if err
        logger.error err.message
        parallelCallback "error on writeFile"
      else
        #config written
        logger.debug "wrote config to weblog config"
        parallelCallback null
      return

  #clear log file
  clearLogFile = (parallelCallback) ->
    fs.writeFile logOutputFilePath, "", (err) ->
      if err
        logger.error err.message
        parallelCallback "error on writeFile"
      else
        logger.debug "Cleared log file"
        parallelCallback null
      return

  restartRsyslog = (seriesCallback) ->
    exec "service rsyslog restart", (error, stdout, stderr) ->
      seriesCallback null unless error
      logger.debug stdout if stdout
      logger.debug stderr if stderr
      if error
        logger.error "exec error: " + error.message
        seriesCallback error
      return

  tail = undefined
  logTimer = undefined

  constructor: ->
    super "gatelogs"
    @sendStatus "stopped"

    #start tailing syslog.olg
    startTailing = (seriesCallback) =>
      tail = spawn "tail", ["-f", logOutputFilePath ]

      handleStdOut = (data) =>
        lines = data.toString("utf-8")
        lines = lines.split("\n")
        lines.pop()
        @sendStatus "running"
        @sendData lines, true
        return

      tail.stdout.on "data", handleStdOut

      tail.on "exit", () =>
        @sendStatus "stopped"
        @stop()

      @sendStatus "started"
      @sendStatus "running"

      seriesCallback null

    @start = (duration) =>
      if not tail or tail.killed
        isRunning = true
        unless duration
          @sendError "Specify a duration!"

        restartAndTail = ->
          async.series [restartRsyslog, startTailing]

        async.parallel [writeNewConfig, clearLogFile], restartAndTail

      else
        logger.warn "gatelogs already running"
        @sendError "gatelogs already running"
        @sendStatus "running"

      # start log, or update duration

      #clear logTimer
      if logTimer
        logger.debug "clearing logger timeout"
        clearTimeout logTimer
      #set logTimer
      logger.debug "setting logger timeout to #{duration} minutes"
      logTimer = setTimeout @stop, duration * 1000 * 60

    @stop = =>
      logger.debug("stop was called")
      if tail and not tail.killed
        tail.kill()
        logger.info "killing tail"

        #clear timer that kills processes after some timeout
        if logTimer
          logger.debug "clearing timeout"
          clearTimeout logTimer

        #delete weblog config
        fs.unlink webLogConfigPath, (err) =>
          if err
            logger.error err.message
            @sendError "Could not delete temporary rsyslog config file: #{err.message}"
            return
          logger.debug "deleted log config" unless err

          restartRsyslog (error) =>
            unless error
              tail = undefined
              # emitter.emit "stopped"
              @sendStatus "stopped"
            @sendError "restarting syslog failed after stopping tail: #{error.message}" if error
            logger.warn "restarting syslog failed after stopping tail: #{error.message}" if error
            isRunning = false

    #read log file and return its content
    @getPreviousLogs = (lines, outputCallback) ->
      thisManyLines = parseInt(lines, 10)
      if thisManyLines < 1
        return

      fs.exists logOutputFilePath, (exists) ->
        # if no logfile exists, send empty array
        unless exists
          outputCallback null, []
          return
        # else, send requested amount of lines
        exec "tail -#{thisManyLines} #{logOutputFilePath}", (error, stdout, stderr) ->
          logger.error error if error
          outputCallback error if error
          logger.warn stderr if stderr
          outputCallback stderr if stderr
          if stdout
            lines = stdout.toString("utf-8")
            lines = lines.split("\n")
            lines.pop()
            outputCallback null, lines

    @deleteLogs = (cb) =>
      weblogFolder = "/tmp/logs/"
      @debug "deleting logs"
      @sendData [], true
      clearFile = (name, asynccb) ->
        fs.exists "#{weblogFolder}/#{name}", (exists) ->
          unless exists
            asynccb null
            return
          fs.writeFile "#{weblogFolder}/#{name}", "", (err) ->
            asynccb(err, "deleted #{name}")

      async.each ["capture.cap", "syslog.olg", "webSyslog.olg"], clearFile, (err) =>
        @sendData [], true
        if err
          @debug "could not delete one file: ", err
        cb(err, [])

  subscribe: =>
    @sendStatus "running" if isRunning
    super()

  getStatus: ->
    if isRunning
      @debug "Status requested, sent 'running': #{isRunning}"
      @sendStatus "running"
    super()

module.exports = new Gatelogs()
